function ensure_success(err)
{
	if (err != SUCCESS)
	{
		switch(err)
		{
			case -202:
				alert("No permission to install files for all users.\n\nUsually you'd have to use an administrator account to install this package.");
				break;
			case -215:
				alert("Could not overwrite existing component.\n\nUsually you'd have to use an administrator account to install this package.");
				break;
		}
		cancelInstall(err);
		throw 'Failed installation';
	}
}

try
{
	const APP_DISPLAY_NAME = "Mozilla Free Desktop Integration";
	const APP_NAME = "freedesktopintegration";
	const APP_PACKAGE = "/mozFreeDesktopIntegration";
	const APP_VERSION = "1.0.2";
	const APP_LOCALES = new Array("en-US");
	const APP_JAR = "mozFreeDesktopIntegration.jar";
	
	initInstall(APP_DISPLAY_NAME, APP_PACKAGE, APP_VERSION);

	var componentsPath = getFolder("Components");
	// we cannot get 'Defaults' directly, as it's no longer supported (it seems)
	var defaultPrefsPath = getFolder("Program", "defaults/pref");

	/*
	  We don't call this variable 'platform' since we're in the
	  'Install' object scope.
	 */
	var isBinaryComponentInstalled = false;
	var ourPlatform = new String(platform);

	if (match = ourPlatform.match(/^X11; Linux .* i(3|4|5|6)86/))
	{
		/* Make sure we don't install a binary object compiled
		   for a higher platform than what we run on. */
		var max_platform_number = match[1];

                for (var i=3; i<=max_platform_number; ++i)
		{
                        var insFileName = "builds/i"+i+"86-linux/mozFreeDesktopIntegration.so";
                        var err = addFile("", APP_VERSION, insFileName, componentsPath, "");

			// DOES_NOT_EXIST error is acceptable at this point, since we're
			// only probing for the file. If none of the probed files are found,
			// we will fail later on.
			if (err != -214)
			{
				ensure_success( err );
				logComment("Installing Linux i"+i+"86 version of component.");
				isBinaryComponentInstalled = true
				break;
			}
                }
	}
	
	if (!isBinaryComponentInstalled && ourPlatform.search(/^X11; SunOS sun4u/) == 0)
	{
		logComment("Installing Sparc Solaris version of component.");
		ensure_success( addFile("", APP_VERSION, "builds/sparc-sun-solaris2.8/mozFreeDesktopIntegration.so", componentsPath, "") );
		isBinaryComponentInstalled = true;
	}

	if (!isBinaryComponentInstalled && ourPlatform.search(/^X11; FreeBSD 5.* i386/) == 0)
	{
		logComment("Installing FreeBSD version of component.");
		ensure_success( addFile("", APP_VERSION, "builds/i386-portbld-freebsd5.2/mozFreeDesktopIntegration.so", componentsPath, "") );
		isBinaryComponentInstalled = true;
	}

	if (!isBinaryComponentInstalled)
	{
		logComment("No version of component for platform (" + ourPlatform + ").");
		alert("This installation package doesn't include a version for your platform\n(" + ourPlatform + ").");
		ensure_success(-201); // Unexpected failure
	}

	// Cannot XPCOM components to profile anyway...
	var isProfile = false;
	var chromeFolder, regFlag = 0;
	if (isProfile)
	{
		chromeFolder = getFolder("Profile", "chrome");
		regFlag |= PROFILE_CHROME;
	}
	else
	{
		chromeFolder = getFolder("chrome");
	}

	// Add XPCOM component
	ensure_success( addFile(null, APP_VERSION, "mozFreeDesktopIntegration-prefs.js", defaultPrefsPath, "") );
	// Add XUL and locale data
	ensure_success( addFile(APP_NAME, APP_JAR, chromeFolder, "") );
	var regSource = getFolder(chromeFolder, APP_JAR);
	ensure_success( registerChrome(PACKAGE | regFlag, regSource, "content/" + APP_NAME + "/") );
	for (var i = 0; i < APP_LOCALES.length; ++i)
	{
		ensure_success( registerChrome(LOCALE | regFlag, regSource, "locale/" + APP_LOCALES[i] + "/" + APP_NAME + "/") );
	}
	ensure_success( performInstall() );
	
	alert(APP_DISPLAY_NAME+" "+APP_VERSION+" has been succesfully installed.\nYou need to restart Mozilla / Thunderbird for the changes to take effect.");
	
	/*
	 This should hopefully refresh XPCOM components, just like navigator.plugins.reload().
	 Unfortunatelly, I cannot use the real XPCOM manager component from here, so we resort
	 to "known hacks".

	 In Mozilla 1.5, it doesn't do that, but it hopefully will in future revisions.
	 */
	refreshPlugins(true);
}
catch (e)
{
	
}
